%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%process_x_y_pos.m
%
%This script gets information about a vessels (x,y) position and counts the
%number of occurences. By doing this, it is possible to get the probability
%of the vessel's postion, p(x,y). This probability will then be used in
%fatigue analysis of a SCR, to see what the effect on the fatigue at the
%TDP.
%
%
%Name: Emir Lejlic
%Date: 16.04.13
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Reading the input-files:

%NB: It is very important to have the folder containing all the x and y
%info in the same directory as this script!

%The input files
x_y_folder = 'X_Y_pos';
x_pos_file = 'Xpos.asc';
y_pos_file = 'Ypos.asc';

%The columns 3-4 are the transformed coord, while columns 5-6 are the 
%transformed are mulitplied in order to reference them into a matrix.
x_y_motion_cols = 6;

%Saving the input into a struct called x_y_motion.

%x_data_rows and x_pos_files give information on how many files there are
%and how large they are. By knowing that it will be easy to loop through
%them.
x_y_motion = read_xy_data(x_y_folder, x_pos_file, y_pos_file, x_y_motion_cols);

%% Finding the max and min values.
[x_pos_files, ~] = size(x_y_motion);

[x_data_rows, ~] = size(x_y_motion(1,1).x_y_data); 

%By knowing the largest values, it will give a good indicator where to
%place my transformed coord-system:

%Want to get the max/min values from col 1 and 2;
index = 1;

[x_max, x_min, y_max, y_min] = ...
 find_extreme_coord(x_y_motion, x_pos_files, index);

fprintf('Max/min values:\n');
fprintf('x_max: %4.2f and x_min: %4.2f\n',x_max,x_min);
fprintf('y_max: %4.2f and y_min: %4.2f\n',y_max,y_min);

fprintf('\nCeiled max and floored min values:\n');
fprintf('x_max: %4.2f and x_min: %4.2f\n',ceil(x_max),floor(x_min));
fprintf('y_max: %4.2f and y_min: %4.2f\n',ceil(y_max),floor(y_min));

%Saving the original max/min into variables:
x_max_orig = x_max;
x_min_orig = x_min;
y_max_orig = y_max;
y_min_orig = y_min;

%Ceiling and flooring the max/min values!
x_max = ceil(x_max);
x_min = floor(x_min);
y_max = ceil(y_max);
y_min = floor(y_min);

%Saving the ceiled/floored max/min values into an array.
rounded_orig_max_values = [x_min x_max y_min y_max];

% Running this function to transform the data.
[x_y_motion] = transform_xy_data(x_y_motion, x_pos_files, x_data_rows,...
    x_min, y_max);

%Want to get the max/min values from the transformed data in col. 3 and 4.
index = 3;
[x_max_trans, x_min_trans, y_max_trans, y_min_trans] = ...
 find_extreme_coord(x_y_motion, x_pos_files, index);

fprintf('\nTransformed max/min values:\n');
fprintf('x_max: %4.2f and x_min: %4.2f\n',x_max_trans,x_min_trans);
fprintf('y_max: %4.2f and y_min: %4.2f\n',y_max_trans,y_min_trans);
fprintf('\nx_min is 0.0 as default. This means that the');
fprintf(' find_extreme_coord function\nwill use this if a lower value is not found!\n');

%The largest values from the transformed
index = 5;
[x_max_trans_100, x_min_trans_100, y_max_trans_100, y_min_trans_100] = ...
 find_extreme_coord(x_y_motion, x_pos_files, index);

%Ceiling and flooring the max/min values!
x_max_trans = ceil(x_max_trans_100);
x_min_trans = floor(x_min_trans_100);
y_max_trans = ceil(y_max_trans_100);
y_min_trans = floor(y_min_trans_100);

fprintf('\nCeiled and floored transformed*100 max/min values:\n');
fprintf('x_max_100: %4.2f and x_min_100: %4.2f\n',...
    x_max_trans_100,x_min_trans_100);
fprintf('y_max_100: %4.2f and y_min_100: %4.2f\n',...
    y_max_trans_100,y_min_trans_100);


% Creating the big coord matrix and counting the occurences of the motion.
clc;

[counted_x_y_data, large_transformed_coord_matrix, total_count] = ...
    count_in_large_matrix(x_y_motion, x_data_rows,...   
    x_pos_files, y_max_trans, x_max_trans, x_min, y_max);
            
%% Inserting the occurences and probability of all x and y coordinates.
clc;

xy_coord_prob = xy_count_and_prob(counted_x_y_data, large_transformed_coord_matrix, total_count, x_min, y_max);

%Checking if all possibilities are in the calc:
clc;
sum_prob = sum(xy_coord_prob(:,4));
sum_count = sum(xy_coord_prob(:,3));

fprintf('\nTotal p(x,y): %4.1f %%.\n',sum_prob*100);
fprintf('Total count: %i occurences.\n',sum_count);

%% Converting the coordinates to polar coordinates.

clc;
xy_polar_cord_prob = convert_xy_to_polar_xy(xy_coord_prob);


%% Counting the position, number of occurences and probability to discrete points

clc;
%The largest radius creates the boundary.
largest_radius = ceil(max(xy_polar_cord_prob(:,3)));
%The angle size of each sector.
sector_degree = 30;
%How many parts each sector has.
sector_parts = 12;

xy_counted_in_sectors = ...
search_xy_in_sectors(xy_polar_cord_prob,sector_degree,largest_radius,sector_parts);

%% Plotting the scatterdata of the discrete points.
copy_to_folder = ...
'C:\Users\Emir Lejlic\Dropbox\Masteroppgave\Master_report\figures\xy_statistics';
%%

close all;

%Discrete points probability
figure()
x = xy_counted_in_sectors(:,1);
y = xy_counted_in_sectors(:,2);
z = xy_counted_in_sectors(:,7);
scatter3(x,y,z,30,'MarkerFaceColor','r');
set(gca,'XTick',-50:15:50)
set(gca,'YTick',-50:15:50)
%axis equal;
grid on;
xlabel('x [m]');
ylabel('y [m]');
zlabel('Probability at (x_i,y_i)');

%This is the way I can create my figures
name = 'discrete_scatter_prob';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%The discrete points from above
figure()
x = xy_counted_in_sectors(:,1);
y = xy_counted_in_sectors(:,2);
z = xy_counted_in_sectors(:,7);
scatter3(x,y,z,30,'MarkerFaceColor','c');
view([0 0 10]);
set(gca,'XTick',-50:10:50)
set(gca,'YTick',-50:10:50)
%axis equal;
grid on;
xlabel('x [m]');
ylabel('y [m]');

%This is the way I can create my figures
name = 'discrete_scatter_prob_Zview';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

% Using the discrete scattered data to create a function Z which will create a surface plot.
%The surface plot is just to have a better picture of how the points are
%connected.
x_min = min(xy_counted_in_sectors(:,1));
x_max = max(xy_counted_in_sectors(:,1));

y_min = min(xy_counted_in_sectors(:,2));
y_max = max(xy_counted_in_sectors(:,2));

%The largest probability.
z_max = max(xy_counted_in_sectors(:,7));

%Creating this for the meshgrid.
xlin = linspace(x_min,x_max,100);
ylin = linspace(y_min,y_max,100);

%Making the grid.
[X,Y] = meshgrid(xlin,ylin);

%The important parameters from the xy_counted_in_sectors.
x = xy_counted_in_sectors(:,1);
y = xy_counted_in_sectors(:,2);
prob = xy_counted_in_sectors(:,7);

%Interpolating the probability at the discrete points.
f = scatteredInterpolant(x,y,prob,'linear','none');

%Z is now the surface function.
Z = f(X,Y);

%% Plotting the surface and contour of the vessel movement probability.
clc;
close all;

figure();
mesh(X,Y,Z) 
hold on
axis([x_min x_max y_min y_max 0 z_max]);
surf(X,Y,Z,'FaceLighting','phong');
view(60,40);
xlabel('x [m]');
ylabel('y [m]');
zlabel('p(x,y) [-]');
shading interp
colorbar;

%This is the way I can create my figures
name = 'surface_discrete_points';
name_pdf = strcat(name,'.pdf');
export_fig(name_pdf,'-pdf','-r300','-transparent','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
clear name name_pdf;

%% The contour of the surface.
close all;
axis_lim =13;

figure()
contourf(X,Y,Z,9)
axis([-axis_lim axis_lim -axis_lim axis_lim]);
xlabel('x [m]');
ylabel('y [m]');
grid on;
legend('p(x,y)');
colorbar;

name = 'contour_of_surface';
name_pdf = strcat(name,'.pdf');
export_fig(name_pdf,'-pdf','-r300','-transparent','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
clear name name_pdf;

%% Scatter plots of all x and y pos.
clc;
close all;

delta = 5;

%Scatter plot of all (x,y) pos. seen from above.
figure()
h_scatter = plot(xy_coord_prob(1:delta:end,1),xy_coord_prob(1:delta:end,2),'.');%,'markersize',12)
hold on;
h_line = line('XData',[0 30],'YData',[0 0],'Color','m','LineWidth',2);
hold on;
h_floater = scatter(0,0,160,'square','MarkerEdgeColor','none','MarkerFaceColor','r');
xlabel('x [m]');
ylabel('y [m]');
legend([h_scatter h_floater h_line],'(x_i,y_i) motion coordinates.', 'Vessel start config.','Riser seen from above.','Location','Best');
%set(gca,'XTick',floor(x_min):5:ceil(x_max))
%set(gca,'YTick',floor(y_min):5:ceil(y_max))
grid on

name = 'scatter_motion_XY';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%% Scatter plot of all (x,y) pos. seen from above.
close all;
delta = 1;

figure()
plot(xy_coord_prob(1:delta:end,1),xy_coord_prob(1:delta:end,2),'.');%,'markersize',12)
axis_lim = 5;
axis([-axis_lim axis_lim -axis_lim axis_lim]);
hold on;
hold on;
h_line = line('XData',[0 30],'YData',[0 0],'Color','m','LineWidth',2);
hold on;
h_floater = scatter(0,0,160,'square','MarkerEdgeColor','none','MarkerFaceColor','r');
% line([0 10],[0 0],'Color','m','LineWidth',2);
% line([0 8.7],[0 5.0],'Color','m','LineWidth',2);
% line([0 5.0],[0 8.7],'Color','m','LineWidth',2);
% line([0 0],[0 10],'Color','m','LineWidth',2);
xlabel('x [m]');
ylabel('y [m]');
%legend('(x_i,y_i) motion coordinates', 'Origin (0,0)','Location','SouthEast','Direction Lines');
set(gca,'XTick',-axis_lim:1:axis_lim)
set(gca,'YTick',-axis_lim:1:axis_lim)
grid on

name = 'scatter_motion_XY_close';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%% Plotting all the data from a speciefied dir folder
clc;
%If I want to check other direction, I want to delete the struct and create
%a new one.
clear plot_2D_struct;

%The direction that is going to be plotted.
direction = 0;

%Creates a struct containing the coordinates for one direction
plot_2D_struct = plot_one_direction_2D(x_y_motion, direction);

Field_names = fieldnames(plot_2D_struct);

close all;
delta = 50;

%Scatter plot of the (x,y) motion.
figure()
hold on
grid on
for i = 1:length(Field_names)
   
    fieldname = genvarname(Field_names{i,1});
    x = plot_2D_struct.(fieldname)(1:delta:end,1);
    y = plot_2D_struct.(fieldname)(1:delta:end,2);
    marker_size = 20;
    scatter(x,y,marker_size,'MarkerEdgeColor','b','MarkerFaceColor','c');
    
        
end
legend(['Direction: ',num2str(direction),'^{\circ}'],'Location','Best');
xlabel('x [m]');
ylabel('y [m]');

name = strcat(['scatter_x_y_motion_',num2str(direction),'_degree']);
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%% Normal plot of the (x,y) motion.
close all;

figure()
hold on
grid on
for i = 1:length(Field_names)
   
    fieldname = genvarname(Field_names{i,1});
    x = plot_2D_struct.(fieldname)(1:delta:end,1);
    y = plot_2D_struct.(fieldname)(1:delta:end,2);
    marker_size = 20;
    plot(x,y);   
        
end
legend(['Direction: ',num2str(direction),' degrees'],'Location','Best');
xlabel('x [m]');
ylabel('y [m]');

name = strcat(['x_y_motion_',num2str(direction),'_degree']);
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;


%% Creating plots of several directions.
clc;
close all;
clear xy_2D_plot_data
%The directions that the analysis are run in:
direction = 0:30:330;

%Creating a struct to save the x and y data that I want.
xy_2D_plot_data(length(direction),1) = struct();

%Number of files in the struct:
no_files = x_pos_files;

% The function
xy_2D_plot_data = x_y_plots_from_above(xy_2D_plot_data, x_y_motion,...
    no_files, direction);

%Saving which run it is into a legend array.
for i = 1:length(direction)
legend_info{i} = xy_2D_plot_data(i,1).name;
end

%Colormap: HSV. There are several to choose from.
cc = jet(length(direction));
%The plot.
figure();
grid on;
hold on;

    for i = 1:length(direction)

        x = xy_2D_plot_data(i,1).x_y_val(:,1);
        y = xy_2D_plot_data(i,1).x_y_val(:,2);
        h_motion = plot(x,y,'color',cc(i,:));       

        clear x y;

    end
hold on;
h_line = line('XData',[0 30],'YData',[0 0],'Color','m','LineWidth',2);
hold on;
h_floater = scatter(0,0,160,'square','MarkerEdgeColor','none','MarkerFaceColor','r');


xlim([-50 30]);
xlabel('x [m]');
ylabel('y [m]');
legend(legend_info,'Location','SouthWest');

name = 'x_y_motion_several_degrees';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%% Creating the variations file:

%Creating a variations file for the (x,y) pos of the vessel. Only the (x,y)
%points with p(x,y) ~= 0.0 are taken into consideration.
counter = 0;
counter_2 = 0;
counter_3 = 0;

prob_limit = 0.00009;

fid = fopen('variations.txt','w');

fprintf(fid,'%s %s %s %s\n','#Id','$x_pos','$y_pos','$prob');

for i = 1:size(xy_counted_in_sectors,1)
   
    if (xy_counted_in_sectors(i,7) >= prob_limit) 
             
        fprintf(fid,'%s %3.1f %3.1f %e\n',strcat('Run_',num2str(counter)),...
                xy_counted_in_sectors(i,1),xy_counted_in_sectors(i,2),...
                xy_counted_in_sectors(i,7));
        
        counter = counter + 1;
        
    elseif (xy_counted_in_sectors(i,7) < prob_limit) && (xy_counted_in_sectors(i,7) > 0)
        
        %Counting between 0 and prob_limit
        counter_2 = counter_2 + 1;
        
    else 
        %Counting which have 0 prob.
        counter_3 = counter_3 + 1;        
        
    end  
      
end

fclose(fid);

num_points_considered = counter;

points_into_consid = zeros(num_points_considered,3);
points_not_into_consid = zeros(counter_2,3);
points_with_zero_prob = zeros(counter_3,3);

%% Plotting which points are taken into account, and which are not.
close all;
clc;

counter_1 = 1;
counter_2 = 1;
counter_3 = 1;

for i = 1:size(xy_counted_in_sectors,1)

    if xy_counted_in_sectors(i,7) >= prob_limit 
       
        points_into_consid(counter_1,1) = xy_counted_in_sectors(i,1);
        points_into_consid(counter_1,2) = xy_counted_in_sectors(i,2);
        points_into_consid(counter_1,3) = xy_counted_in_sectors(i,7);
        counter_1 = counter_1 + 1;
        
    elseif (xy_counted_in_sectors(i,7) < prob_limit) && (xy_counted_in_sectors(i,7) > 0)
        
        points_not_into_consid(counter_2,1) = xy_counted_in_sectors(i,1);
        points_not_into_consid(counter_2,2) = xy_counted_in_sectors(i,2);
        points_not_into_consid(counter_2,3) = xy_counted_in_sectors(i,7);
        counter_2 = counter_2 + 1;
        
    else
        
        points_with_zero_prob(counter_3,1) = xy_counted_in_sectors(i,1);
        points_with_zero_prob(counter_3,2) = xy_counted_in_sectors(i,2);
        points_with_zero_prob(counter_3,3) = xy_counted_in_sectors(i,7);
        counter_3 = counter_3 + 1;
        
    end    

end

fprintf('Sum of p(x,y) into account: %f %%\n',100*sum(points_into_consid(:,3)));
fprintf('Sum of p(x,y) not into account: %f %%\n',100*sum(points_not_into_consid(:,3)));

figure()
grid on;
scatter(points_into_consid(:,1),points_into_consid(:,2),30,'MarkerFaceColor','b','MarkerEdgeColor','none');
%axis equal;
hold on;
grid on;
scatter(points_not_into_consid(:,1),points_not_into_consid(:,2),30,'MarkerFaceColor','r','MarkerEdgeColor','none');
hold on;
grid on;
scatter(points_with_zero_prob(:,1),points_with_zero_prob(:,2),30,'MarkerFaceColor','g','MarkerEdgeColor','none');
set(gca,'XTick',-50:10:50)
title(['Total probability of the blue points: ',num2str(100*sum(points_into_consid(:,3)),3),' %']);
xlabel('x [m]');
ylabel('y [m]');
legend(['p(x_i,y_i) >= ',num2str(prob_limit,'%6.5f')],['p(x_i,y_i) < ',...
    num2str(prob_limit,'%6.5f')],'p(x_i,y_i) = 0.0','Orientation','Horizontal','Location',...
    'NorthOutside');

name = 'xy_points_into_consideration';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;

%%
clc;
close all;
%The values where the coordinates will be split up between to get a smaller
%plot. There are many values between the inner and outer radius, and they
%need to be decreased without losing the info near the origin and close to
%the ends.
%The inner and outer radius
inner_radius = 8;
outer_radius = 30;

%The interval jumper in the between coords. 
%Ex: xy_coord_between(1:delta_delete:end) = [];
delta_delete = 2;

%The filtered_motion_matrix is used to show the plot of a large number of
%the xy coordinates.
filtered_xy_motion = plot_raw_xy_motion(xy_coord_prob, inner_radius, outer_radius, delta_delete);

%Giving new names to make it easier to plot...
x = filtered_xy_motion(:,1);
y = filtered_xy_motion(:,2);
prob = filtered_xy_motion(:,3);

%Maybe needed
delta = 5;

%Examples of points with no occurences
x_points = [-20 -10 -30 -30];
y_points = [-20 -30 -10 10];

figure()
scatter3(x(1:delta:end),y(1:delta:end),prob(1:delta:end),10,'b');
hold on
scatter(x_points,y_points,'r','fill');
xlabel('x [m]');
ylabel('y [m]');
zlabel('p(x_i,y_i)');
legend('Prob. at vessel motion coordinate (x_i,y_i).','Points in areas with no occurences.',...
    'Location','Best');

name = 'prob_all_points';
name_pdf = strcat(name,'.pdf');
name_eps = strcat(name,'.eps');
export_fig(name_pdf,'-pdf','-transparent','-painters');
print(gcf,name_eps,'-depsc2','-painters');

copyfile(name_pdf,copy_to_folder);
delete(name_pdf);
copyfile(name_eps,copy_to_folder);
delete(name_eps);
clear name name_eps name_pdf;